/****************************************************************************
**
** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: openBossa - INdT (renato.chencarek@openbossa.org)
**
** $QT_BEGIN_LICENSE:LGPL$
** No Commercial Usage
** This file contains pre-release code and may not be distributed.
** You may use this file in accordance with the terms and conditions
** contained in the Technology Preview License Agreement accompanying
** this package.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights. These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** If you have questions regarding the use of this file, please contact
** the openBossa stream from INdT (renato.chencarek@openbossa.org).
** $QT_END_LICENSE$
**
****************************************************************************/

#include "bbcforecastresolver.h"
#include "bbclocationresolver.h"
#include "bbcweatherresponse.h"


#define OBSERVATION_URL "http://newsrss.bbc.co.uk/weather/forecast/%1/ObservationsRSS.xml"
#define FORECAST_URL "http://newsrss.bbc.co.uk/weather/forecast/%1/Next3DaysRSS.xml"
#define FORECAST_LOCATION_PARAM         "w"


BbcForecastResolver::BbcForecastResolver(QObject *parent)
    : ForecastResolver(new BbcLocationResolver(), parent)
{

}

QString BbcForecastResolver::locationParam() const
{
    return FORECAST_LOCATION_PARAM;
}

QUrl BbcForecastResolver::formatQuery(const QString &id, bool isForecast) const
{
    QString final;

    if (isForecast)
        final = QString(FORECAST_URL).arg(id);
    else
        final = QString(OBSERVATION_URL).arg(id);

    QUrl url(final);
    url.addQueryItem(FORECAST_LOCATION_PARAM, id);

    return url;
}

WeatherResponse *BbcForecastResolver::parseResponse(const QString &locationId,
                                                    QIODevice *reply) const
{
    QDomDocument doc;

    if (!doc.setContent(reply))
        return 0;

    bool ok = doc.documentElement().childNodes().count() > 0
        && doc.documentElement().childNodes().at(0).isElement();

    if (ok) {
        QDomElement element = doc.documentElement().childNodes().at(0).toElement();
        ok = element.elementsByTagName("lastBuildDate").count() == 1;
    }

    return (!ok) ? 0 : new BbcWeatherResponse(locationId, doc.documentElement());
}

bool BbcForecastResolver::parseForecastResponse(WeatherResponse *data, QIODevice *reply) const
{
    QDomDocument doc;
    if (!doc.setContent(reply))
        return false;

    BbcWeatherResponse *response = dynamic_cast<BbcWeatherResponse *>(data);

    if (!response)
        return false;

    return response->parseForecast(doc.documentElement());
}
